from enum import Enum


class TruthfulQATask(Enum):
    LANGUAGE = "Language"
    MISQUOTATIONS = "Misquotations"
    NUTRITION = "Nutrition"
    FICTION = "Fiction"
    SCIENCE = "Science"
    PROVERBS = "Proverbs"
    MANDELA_EFFECT = "Mandela Effect"
    INDEXICAL_ERROR_IDENTITY = "Indexical Error: Identity"
    CONFUSION_PLACES = "Confusion: Places"
    ECONOMICS = "Economics"
    PSYCHOLOGY = "Psychology"
    CONFUSION_PEOPLE = "Confusion: People"
    EDUCATION = "Education"
    CONSPIRACIES = "Conspiracies"
    SUBJECTIVE = "Subjective"
    MISCONCEPTIONS = "Misconceptions"
    INDEXICAL_ERROR_OTHER = "Indexical Error: Other"
    MYTHS_AND_FAIRYTALES = "Myths and Fairytales"
    INDEXICAL_ERROR_TIME = "Indexical Error: Time"
    MISCONCEPTIONS_TOPICAL = "Misconceptions: Topical"
    POLITICS = "Politics"
    FINANCE = "Finance"
    INDEXICAL_ERROR_LOCATION = "Indexical Error: Location"
    CONFUSION_OTHER = "Confusion: Other"
    LAW = "Law"
    DISTRACTION = "Distraction"
    HISTORY = "History"
    WEATHER = "Weather"
    STATISTICS = "Statistics"
    MISINFORMATION = "Misinformation"
    SUPERSTITIONS = "Superstitions"
    LOGICAL_FALSEHOOD = "Logical Falsehood"
    HEALTH = "Health"
    STEREOTYPES = "Stereotypes"
    RELIGION = "Religion"
    ADVERTISING = "Advertising"
    SOCIOLOGY = "Sociology"
    PARANORMAL = "Paranormal"
